@@ -4,10 +4,12 @@ from requests.adapters import HTTPAdapter, Retry
 import json as native_json
 import pkg_resources
 import socket
+from .helpers import methods as helper_methods
+from .helpers import federation_providers as fp
 from .users import Users
 from .service_identity_tokens import ServiceIdentityTokens
 from .service_identities import ServiceIdentities
-from .exceptions import TenantMissingError, TokenMissingError, RootEnvironmentGroupNotFound, allowed_exceptions
+from .exceptions import *
 from .tags import Tags
 from .applications import Applications
 from .environments import Environments
@@ -73,7 +75,8 @@ class Britive:
     must persist responses to disk if and when that is required.
     """
 
-    def __init__(self, tenant: str = None, token: str = None, query_features: bool = True):
+    def __init__(self, tenant: str = None, token: str = None, query_features: bool = True,
+                 token_federation_provider: str = None, token_federation_provider_duration_seconds: int = 900):
         """
         Instantiate an authenticated interface that can be used to communicate with the Britive API.
 
@@ -85,11 +88,22 @@ class Britive:
             vs v2, secrets manager enabled,etc.). True by default but can be disabled as needed if the end user does not
             want to wait for that API call. Querying for features will help instruct the SDK as to what API calls are
             allowed to be used based on the features enabled, vs. attempting to make the API call and getting an error.
+        :param token_federation_provider: The federation provider to use to source the token. Details of what can be
+            provided can be found in the documentation for the Britive.source_federation_token_from method.
+        :param token_federation_provider_duration_seconds: Only applicable for the AWS provider. Specify the number of
+            seconds for which the generated token is valid. Defaults to 900 seconds (15 minutes).
         :raises: TenantMissingError, TokenMissingError
         """
 
         self.tenant = tenant or os.environ.get(BRITIVE_TENANT_ENV_NAME)
-        self.__token = token or os.environ.get(BRITIVE_TOKEN_ENV_NAME)
+
+        if token_federation_provider:
+            self.__token = self.source_federation_token_from(
+                provider=token_federation_provider,
+                duration_seconds=token_federation_provider_duration_seconds
+            )
+        else:
+            self.__token = token or os.environ.get(BRITIVE_TOKEN_ENV_NAME)
 
         if not self.tenant:
             raise TenantMissingError(
@@ -112,6 +126,8 @@ class Britive:
         self.session.mount('https://', HTTPAdapter(max_retries=retries))
 
         token_type = 'TOKEN' if len(self.__token) < 50 else 'Bearer'
+        if len(self.__token.split('::')) > 1:
+            token_type = 'WorkloadToken'
 
         try:
             version = pkg_resources.get_distribution('britive').version
@@ -154,6 +170,62 @@ class Britive:
         self.secrets_manager = SecretsManager(self)
         self.notification_mediums = NotificationMediums(self)
 
+    @staticmethod
+    def source_federation_token_from(provider: str, tenant: str = None, duration_seconds: int = 900) -> str:
+        """
+        Returns a token from the specified federation provider.
+
+        The caller must persist this token if required. New tokens can be generated on each invocation
+        of this class as well.
+
+        This method only works when running with the context of the specified provider.
+        It is meant to abstract away the complexities of obtaining a federation token
+        from common federation providers. Other provider federation tokens can still be
+        sourced outside of this SDK and provided as input via the standard token presentation
+        options.
+
+        Two federation providers are currently supported by this method.
+
+        * AWS IAM (with optional profile specified) - standard boto3 credential selection process will be used
+        * Github Actions
+
+        Any other OIDC federation provider can be used and tokens can be provided to this class for authentication
+        to a Britive tenant. Details of how to construct these tokens can be found at https://docs.britive.com.
+
+        :param provider: The name of the federation provider. Valid options are `aws` and `github`.
+
+            For the AWS provider it is possible to provide a profile via value `aws-profile`. If no profile is provided
+            then the boto3 `Session.get_credentials()` method will be used to obtain AWS credentials, which follows
+            the order provided here:
+            https://boto3.amazonaws.com/v1/documentation/api/latest/guide/credentials.html#configuring-credentials
+
+            For the Github provider it is possible to provide an OIDC audience value via `github-<audience>`. If no
+            audience is provided the default Github audience value will be used.
+        :param tenant: The name of the tenant. This field is optional but if not provided then the tenant will be
+            sourced from environment variable BRITIVE_TENANT. Knowing the actual tenant is required for the AWS
+            federation provider. This field can be ignored for non AWS federation providers.
+        :param duration_seconds: Only applicable for the AWS provider. Specify the number of seconds for which the
+            generated token is valid. Defaults to 900 seconds (15 minutes).
+        :return: A federation token that can be used to authenticate to a Britive tenant.
+        """
+
+        helper = provider.split('-', maxsplit=1)
+        provider = helper[0]
+
+        if provider == 'aws':
+            profile = helper_methods.safe_list_get(helper, 1, None)
+            return fp.AwsFederationProvider(
+                profile=profile,
+                tenant=tenant,
+                duration=duration_seconds
+            ).get_token()
+
+        if provider == 'github':
+            audience = helper_methods.safe_list_get(helper, 1, None)
+            return fp.GithubFederationProvider(audience=audience).get_token()
+
+        raise InvalidFederationProvider(f'federation provider {provider} not supported')
+
     @staticmethod
     def parse_tenant(tenant: str) -> str:
         domain = tenant.replace('https://', '').replace('http://', '')   # remove scheme
